/*
/*  ActionController.m
/*
 IMPORTANT:  This sample software is supplied to you by Blacksmith Technologies, Inc. ("Blacksmith") in consideration of your agreement to the following terms.  Your use, installation,  modification or redistribution of this software constitutes acceptance of these terms.  If you do not agree with these terms, please do not use, install, modify or redistribute this software.
  
This software is provided by Blacksmith on an "AS IS" basis.  Blacksmith makes no warranties, express or implied, including without limitation the implied warranties of non-infringement, merchantability and fitness for a particular purpose, regarding this software or its use and operation alone or in combination with your products.  In no event shall Blacksmith be liable for any special, indirect, incidental or consequential damages arising in any way out of the use, reproduction, modification and/or distribution of this software, however caused and whether under theory of contract, tort, strict liability or otherwise, even if Blacksmith has been advised of the possibility of such damage.

*/


#import "ActionController.h"
#import "DataManager.h"
#import "ChartBuilder.h"
#import "ChartsmithInterface.h"

@implementation ActionController

- (void)setStatus:(double)status message:(NSString*)message
{
    /* update the status displayed on window */
    [statusMessage setStringValue:message]; 
    [statusMessage displayIfNeeded];
    [progressIndicator setDoubleValue:status];
    [progressIndicator displayIfNeeded];
}

- (BOOL)validateChartsmith
{
    /* ensure that Chartsmith can be used (i.e. that it is licensed) - Note that 
    /* the only reason Chartsmith must be licensed is for the image exporting function,
    /* all other commands used in this app can be sent to Chartsmith without it being licensed */
    ChartsmithInterface *interface = [ChartsmithInterface sharedChartsmithInterface];
    if( ![interface isChartsmithLicensed] ){
        NSRunAlertPanel(@"Contact chartsmith@blacksmith.com",@"This sample application cannot be executed until Chartsmith is licensed.  Contact Blacksmith for a free expiring evaluation license: chartsmith@blacksmith.com.",@"OK",@"",nil);
        return NO;
    }
    return YES;
}

- (BOOL)validateDates:(NSCalendarDate*)fromDate :(NSCalendarDate*)toDate
{
    NSCalendarDate *oneWeekPastFromDate, *oneMonthPastFromDate; 

    /* ensure dates are ordered correctly */
    if( [toDate compare:fromDate]==NSOrderedAscending ){
        NSRunAlertPanel(@"Invalid dates",@"Starting date must be before ending date.",@"OK",@"",nil);
        return NO;
    }
    
    /* ensure a maximum of 6 months of data  */
    oneMonthPastFromDate = [fromDate dateByAddingYears:0 months:6 days:0 hours:0 minutes:0 seconds:0];
    if( [oneMonthPastFromDate compare:toDate]==NSOrderedAscending ){
        NSRunAlertPanel(@"Invalid dates",@"Duration of data collected can be no more than 6 months.",@"OK",@"",nil);
        return NO;
    }
    
    /* be certain we're collecting at least one week of data - the Yahoo 
    /* interface seems to return erroneous data if it's less than a week */
    oneWeekPastFromDate = [fromDate dateByAddingYears:0 months:0 days:7 hours:0 minutes:0 seconds:0];
    if( [oneWeekPastFromDate compare:toDate]==NSOrderedDescending ){
        NSRunAlertPanel(@"Invalid dates",@"Duration of data collected must be at least one week long.",@"OK",@"",nil);
        return NO;
    }

    return YES;
}

- (IBAction)generateChart:sender
{
    /* this method is messaged from the "Chart It" button on the UI */
    DataManager *dataManager = [[[DataManager alloc] init] autorelease];
    ChartBuilder *chartBuilder = [[[ChartBuilder alloc] init] autorelease];
    NSArray	*dataSet;
    NSString *imagePath = @"/tmp/scriptchart.pdf";
    NSCalendarDate *fromDate, *toDate;

    if( ![self validateChartsmith] )
        return;
        
    /* get start and end dates from user interface */
    fromDate = [NSCalendarDate 
        dateWithYear:[[[fromDateYear selectedItem] title] intValue] 
        month:[[fromDateMonth selectedItem] tag] 
        day:[[[fromDateDay selectedItem] title] intValue] 
        hour:0 minute:0 second:0 
        timeZone:[NSTimeZone timeZoneWithAbbreviation:@"EST"]];
    toDate = [NSCalendarDate 
        dateWithYear:[[[toDateYear selectedItem] title] intValue] 
        month:[[toDateMonth selectedItem] tag] 
        day:[[[toDateDay selectedItem] title] intValue] 
        hour:0 minute:0 second:0 
        timeZone:[NSTimeZone timeZoneWithAbbreviation:@"EST"]];
    
    if( ![self validateDates:fromDate :toDate] ){
        [self setStatus:0 message:@""];
        return;
    }
    
    /* retrieve data */
	[self setStatus:10 message:@"Retrieving data..."];
	dataSet = [dataManager retrieveDataForSymbol:[tickerText stringValue] fromDate:fromDate toDate:toDate];
    
    if( !dataSet || ![dataSet count] ){
        NSRunAlertPanel(@"Error retrieving data",@"This is likely due to the network connection being unavailable, or the ticker symbol being invalid.",@"OK",@"",nil);
        [self setStatus:0 message:@""];
        return;
    }
        
    /* build price chart */
	[self setStatus:30 message:@"Constucting price chart..."];
    [chartBuilder buildPriceChartFromDataSet:dataSet atChartIndex:PRICE_CHART];
    [chartBuilder setChartTitle:[NSString stringWithFormat:@"Ticker Symbol: %@",[tickerText stringValue]] subTitle:[NSString stringWithFormat:@"From %@ to %@",[fromDate descriptionWithCalendarFormat:@"%m/%d/%y"],[toDate descriptionWithCalendarFormat:@"%m/%d/%y"]] forChart:PRICE_CHART];
    
    /* display price chart */
	[self setStatus:50 message:@"Displaying price chart..."];
    [chartBuilder exportChartToPath:imagePath forChart:PRICE_CHART];
    [priceView setImage:[[[NSImage alloc] initWithContentsOfFile:imagePath] autorelease]];

    /* build volume chart */
	[self setStatus:70 message:@"Constructing volume chart..."];
    [chartBuilder buildVolumeChartFromDataSet:dataSet atChartIndex:VOLUME_CHART];
    
    /* display volume chart */
	[self setStatus:90 message:@"Displaying volume chart..."];
    [chartBuilder exportChartToPath:imagePath forChart:VOLUME_CHART];
    [volumeView setImage:[[[NSImage alloc] initWithContentsOfFile:imagePath] autorelease]];

	[self setStatus:0 message:@""];
}

@end
